@@ -499,6 +499,7 @@ TridentInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 
     /* Enable Chipset specific options */
     switch (pTrident->Chipset) {
+	case XP5:
 	case CYBERBLADEXP4:
 	case CYBERBLADEXPAI1:
 	case BLADEXP:
@@ -574,6 +575,7 @@ TridentInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     	    pReg->tridentRegs3x4[PixelBusReg] = 0x29;
 	    pReg->tridentRegsDAC[0x00] = 0xD0;
 	    if (pTrident->Chipset == CYBERBLADEXP4 ||
+	        pTrident->Chipset == XP5 ||
 	        pTrident->Chipset == CYBERBLADEE4) {
     		OUTB(vgaIOBase+ 4, New32);
 		pReg->tridentRegs3x4[New32] = INB(vgaIOBase + 5) & 0x7F;
@@ -582,6 +584,7 @@ TridentInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	case 32:
 	    pReg->tridentRegs3CE[MiscExtFunc] |= 0x02;
 	    if (pTrident->Chipset != CYBERBLADEXP4
+	        && pTrident->Chipset != XP5
 	        && pTrident->Chipset != CYBERBLADEE4
 		&& pTrident->Chipset != CYBERBLADEXPAI1) {
 	        /* Clock Division by 2*/
@@ -592,6 +595,7 @@ TridentInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     	    pReg->tridentRegs3x4[PixelBusReg] = 0x09;
 	    pReg->tridentRegsDAC[0x00] = 0xD0;
 	    if (pTrident->Chipset == CYBERBLADEXP4
+	        || pTrident->Chipset == XP5
 	        || pTrident->Chipset == CYBERBLADEE4
 		|| pTrident->Chipset == CYBERBLADEXPAI1) {
     		OUTB(vgaIOBase+ 4, New32);
@@ -721,7 +725,8 @@ TridentInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     	OUTB(0x3C5, protect);
     }
  
-    if (pTrident->Chipset == CYBERBLADEXP4)
+    if (pTrident->Chipset == CYBERBLADEXP4 ||
+        pTrident->Chipset == XP5)
     	pReg->tridentRegs3CE[DisplayEngCont] = 0x08;
 
     /* Avoid lockup on Blade3D, PCI Retry is permanently on */
@@ -800,8 +805,10 @@ TridentRestore(ScrnInfoPtr pScrn, TRIDENTRegPtr tridentReg)
     if (pTrident->Chipset >= CYBER9385)    OUTW_3x4(Enhancement0);
     if (pTrident->Chipset >= BLADE3D)      OUTW_3x4(RAMDACTiming);
     if (pTrident->Chipset == CYBERBLADEXP4 ||
+        pTrident->Chipset == XP5 ||
         pTrident->Chipset == CYBERBLADEE4) OUTW_3x4(New32);
-    if (pTrident->Chipset == CYBERBLADEXP4) OUTW_3CE(DisplayEngCont);
+    if (pTrident->Chipset == CYBERBLADEXP4 ||
+        pTrident->Chipset == XP5) OUTW_3CE(DisplayEngCont);
     if (pTrident->IsCyber) {
 	CARD8 tmp;
 
@@ -932,8 +939,10 @@ TridentSave(ScrnInfoPtr pScrn, TRIDENTRegPtr tridentReg)
     if (pTrident->Chipset >= CYBER9385)    INB_3x4(Enhancement0);
     if (pTrident->Chipset >= BLADE3D)      INB_3x4(RAMDACTiming);
     if (pTrident->Chipset == CYBERBLADEXP4 ||
+        pTrident->Chipset == XP5 ||
         pTrident->Chipset == CYBERBLADEE4) INB_3x4(New32);
-    if (pTrident->Chipset == CYBERBLADEXP4) INB_3CE(DisplayEngCont);
+    if (pTrident->Chipset == CYBERBLADEXP4 ||
+        pTrident->Chipset == XP5) INB_3CE(DisplayEngCont);
     if (pTrident->IsCyber) {
 	CARD8 tmp;
 	INB_3CE(VertStretch);
@@ -1168,6 +1177,7 @@ TridentHWCursorInit(ScreenPtr pScreen)
 		HARDWARE_CURSOR_SWAP_SOURCE_AND_MASK |
 		HARDWARE_CURSOR_SOURCE_MASK_INTERLEAVE_32 |
                 ((pTrident->Chipset == CYBERBLADEXP4 ||
+                  pTrident->Chipset == XP5 ||
                   pTrident->Chipset == CYBERBLADEE4) ? 
                 HARDWARE_CURSOR_TRUECOLOR_AT_8BPP : 0);
     infoPtr->SetCursorColors = TridentSetCursorColors;
